"""Generated message classes for batch version v1.

An API to manage the running of batch jobs on Google Cloud Platform.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'batch'


class Accelerator(_messages.Message):
  r"""Accelerator describes Compute Engine accelerators to be attached to the
  VM.

  Fields:
    count: The number of accelerators of this type.
    installGpuDrivers: Deprecated: please use instances[0].install_gpu_drivers
      instead.
    type: The accelerator type. For example, "nvidia-tesla-t4". See `gcloud
      compute accelerator-types list`.
  """

  count = _messages.IntegerField(1)
  installGpuDrivers = _messages.BooleanField(2)
  type = _messages.StringField(3)


class ActionCondition(_messages.Message):
  r"""Conditions for actions to deal with task failures.

  Fields:
    exitCodes: Exit codes of a task execution. If there are more than 1 exit
      codes, when task executes with any of the exit code in the list, the
      condition is met and the action will be executed.
  """

  exitCodes = _messages.IntegerField(1, repeated=True, variant=_messages.Variant.INT32)


class AllocationPolicy(_messages.Message):
  r"""A Job's resource allocation policy describes when, where, and how
  compute resources should be allocated for the Job.

  Messages:
    LabelsValue: Labels applied to all VM instances and other resources
      created by AllocationPolicy. Labels could be user provided or system
      generated. You can assign up to 64 labels. [Google Compute Engine label
      restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) apply. Label names that start with "goog-" or
      "google-" are reserved.

  Fields:
    instances: Describe instances that can be created by this
      AllocationPolicy. Only instances[0] is supported now.
    labels: Labels applied to all VM instances and other resources created by
      AllocationPolicy. Labels could be user provided or system generated. You
      can assign up to 64 labels. [Google Compute Engine label
      restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) apply. Label names that start with "goog-" or
      "google-" are reserved.
    location: Location where compute resources should be allocated for the
      Job.
    network: The network policy.
    serviceAccount: Service account that VMs will run as.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels applied to all VM instances and other resources created by
    AllocationPolicy. Labels could be user provided or system generated. You
    can assign up to 64 labels. [Google Compute Engine label
    restrictions](https://cloud.google.com/compute/docs/labeling-
    resources#restrictions) apply. Label names that start with "goog-" or
    "google-" are reserved.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  instances = _messages.MessageField('InstancePolicyOrTemplate', 1, repeated=True)
  labels = _messages.MessageField('LabelsValue', 2)
  location = _messages.MessageField('LocationPolicy', 3)
  network = _messages.MessageField('NetworkPolicy', 4)
  serviceAccount = _messages.MessageField('ServiceAccount', 5)


class AttachedDisk(_messages.Message):
  r"""A new or an existing persistent disk (PD) or a local ssd attached to a
  VM instance.

  Fields:
    deviceName: Device name that the guest operating system will see. It is
      used by Runnable.volumes field to mount disks. So please specify the
      device_name if you want Batch to help mount the disk, and it should
      match the device_name field in volumes.
    existingDisk: Name of an existing PD.
    newDisk: A Disk attribute.
  """

  deviceName = _messages.StringField(1)
  existingDisk = _messages.StringField(2)
  newDisk = _messages.MessageField('Disk', 3)


class Barrier(_messages.Message):
  r"""Barrier runnable blocks until all tasks in a taskgroup reach it.

  Fields:
    name: Barriers are identified by their index in runnable list. Names are
      not required, but if present should be an identifier.
  """

  name = _messages.StringField(1)


class BatchProjectsLocationsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsCreateRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsCreateRequest object.

  Fields:
    job: A Job resource to be passed as the request body.
    jobId: ID used to uniquely identify the Job within its parent scope. This
      field should contain at most 63 characters and must start with lowercase
      characters. Only lowercase characters, numbers and '-' are accepted. The
      '-' character cannot be the first or the last one. A system generated ID
      will be used if the field is not set. The job.name field in the request
      will be ignored and the created resource name of the Job will be
      "{parent}/jobs/{job_id}".
    parent: Required. The parent resource name where the Job will be created.
      Pattern: "projects/{project}/locations/{location}"
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  job = _messages.MessageField('Job', 1)
  jobId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class BatchProjectsLocationsJobsDeleteRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsDeleteRequest object.

  Fields:
    name: Job name.
    reason: Optional. Reason for this deletion.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  reason = _messages.StringField(2)
  requestId = _messages.StringField(3)


class BatchProjectsLocationsJobsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsGetRequest object.

  Fields:
    name: Required. Job name.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsListRequest object.

  Fields:
    filter: List filter.
    pageSize: Page size.
    pageToken: Page token.
    parent: Parent path.
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BatchProjectsLocationsJobsTaskGroupsTasksGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsTaskGroupsTasksGetRequest object.

  Fields:
    name: Required. Task name.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsJobsTaskGroupsTasksListRequest(_messages.Message):
  r"""A BatchProjectsLocationsJobsTaskGroupsTasksListRequest object.

  Fields:
    filter: Task filter, null filter matches all Tasks. Filter string should
      be of the format State=TaskStatus.State e.g. State=RUNNING
    pageSize: Page size.
    pageToken: Page token.
    parent: Required. Name of a TaskGroup from which Tasks are being
      requested. Pattern: "projects/{project}/locations/{location}/jobs/{job}/
      taskGroups/{task_group}"
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class BatchProjectsLocationsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsListRequest object.

  Fields:
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class BatchProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class BatchProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class BatchProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A BatchProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class ComputeResource(_messages.Message):
  r"""Compute resource requirements

  Fields:
    bootDiskMib: Extra boot disk size in MiB for each task.
    cpuMilli: The milliCPU count.
    memoryMib: Memory in MiB.
  """

  bootDiskMib = _messages.IntegerField(1)
  cpuMilli = _messages.IntegerField(2)
  memoryMib = _messages.IntegerField(3)


class Container(_messages.Message):
  r"""Container runnable.

  Fields:
    blockExternalNetwork: If set to true, external network access to and from
      container will be blocked, containers that are with
      block_external_network as true can still communicate with each other,
      network cannot be specified in the `container.options` field.
    commands: Overrides the `CMD` specified in the container. If there is an
      ENTRYPOINT (either in the container image or with the entrypoint field
      below) then commands are appended as arguments to the ENTRYPOINT.
    entrypoint: Overrides the `ENTRYPOINT` specified in the container.
    imageUri: The URI to pull the container image from.
    options: Arbitrary additional options to include in the "docker run"
      command when running this container, e.g. "--network host".
    password: Optional password for logging in to a docker registry. If
      password matches `projects/*/secrets/*/versions/*` then Batch will read
      the password from the Secret Manager;
    username: Optional username for logging in to a docker registry. If
      username matches `projects/*/secrets/*/versions/*` then Batch will read
      the username from the Secret Manager.
    volumes: Volumes to mount (bind mount) from the host machine files or
      directories into the container, formatted to match docker run's --volume
      option, e.g. /foo:/bar, or /foo:/bar:ro
  """

  blockExternalNetwork = _messages.BooleanField(1)
  commands = _messages.StringField(2, repeated=True)
  entrypoint = _messages.StringField(3)
  imageUri = _messages.StringField(4)
  options = _messages.StringField(5)
  password = _messages.StringField(6)
  username = _messages.StringField(7)
  volumes = _messages.StringField(8, repeated=True)


class Disk(_messages.Message):
  r"""A new persistent disk or a local ssd. A VM can only have one local SSD
  setting but multiple local SSD partitions. See
  https://cloud.google.com/compute/docs/disks#pdspecs and
  https://cloud.google.com/compute/docs/disks#localssds.

  Fields:
    diskInterface: Local SSDs are available through both "SCSI" and "NVMe"
      interfaces. If not indicated, "NVMe" will be the default one for local
      ssds. We only support "SCSI" for persistent disks now.
    image: Name of a public or custom image used as the data source. For
      example, the following are all valid URLs: * Specify the image by its
      family name: projects/{project}/global/images/family/{image_family} *
      Specify the image version:
      projects/{project}/global/images/{image_version} You can also use Batch
      customized image in short names. The following image values are
      supported for a boot disk: * "batch-debian": use Batch Debian images. *
      "batch-centos": use Batch CentOS images. * "batch-cos": use Batch
      Container-Optimized images.
    sizeGb: Disk size in GB. For persistent disk, this field is ignored if
      `data_source` is `image` or `snapshot`. For local SSD, size_gb should be
      a multiple of 375GB, otherwise, the final size will be the next greater
      multiple of 375 GB. For boot disk, Batch will calculate the boot disk
      size based on source image and task requirements if you do not speicify
      the size. If both this field and the boot_disk_mib field in task spec's
      compute_resource are defined, Batch will only honor this field.
    snapshot: Name of a snapshot used as the data source. Snapshot is not
      supported as boot disk now.
    type: Disk type as shown in `gcloud compute disk-types list`. For example,
      local SSD uses type "local-ssd". Persistent disks and boot disks use
      "pd-balanced", "pd-extreme", "pd-ssd" or "pd-standard".
  """

  diskInterface = _messages.StringField(1)
  image = _messages.StringField(2)
  sizeGb = _messages.IntegerField(3)
  snapshot = _messages.StringField(4)
  type = _messages.StringField(5)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class Environment(_messages.Message):
  r"""An Environment describes a collection of environment variables to set
  when executing Tasks.

  Messages:
    SecretVariablesValue: A map of environment variable names to Secret
      Manager secret names. The VM will access the named secrets to set the
      value of each environment variable.
    VariablesValue: A map of environment variable names to values.

  Fields:
    encryptedVariables: An encrypted JSON dictionary where the key/value pairs
      correspond to environment variable names and their values.
    secretVariables: A map of environment variable names to Secret Manager
      secret names. The VM will access the named secrets to set the value of
      each environment variable.
    variables: A map of environment variable names to values.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class SecretVariablesValue(_messages.Message):
    r"""A map of environment variable names to Secret Manager secret names.
    The VM will access the named secrets to set the value of each environment
    variable.

    Messages:
      AdditionalProperty: An additional property for a SecretVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type SecretVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a SecretVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class VariablesValue(_messages.Message):
    r"""A map of environment variable names to values.

    Messages:
      AdditionalProperty: An additional property for a VariablesValue object.

    Fields:
      additionalProperties: Additional properties of type VariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a VariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  encryptedVariables = _messages.MessageField('KMSEnvMap', 1)
  secretVariables = _messages.MessageField('SecretVariablesValue', 2)
  variables = _messages.MessageField('VariablesValue', 3)


class GCS(_messages.Message):
  r"""Represents a Google Cloud Storage volume.

  Fields:
    remotePath: Remote path, either a bucket name or a subdirectory of a
      bucket, e.g.: bucket_name, bucket_name/subdirectory/
  """

  remotePath = _messages.StringField(1)


class InstancePolicy(_messages.Message):
  r"""InstancePolicy describes an instance type and resources attached to each
  VM created by this InstancePolicy.

  Enums:
    ProvisioningModelValueValuesEnum: The provisioning model.

  Fields:
    accelerators: The accelerators attached to each VM instance.
    bootDisk: Boot disk to be created and attached to each VM by this
      InstancePolicy. Boot disk will be deleted when the VM is deleted. Batch
      API now only supports booting from image.
    disks: Non-boot disks to be attached for each VM created by this
      InstancePolicy. New disks will be deleted when the VM is deleted.
    machineType: The Compute Engine machine type.
    minCpuPlatform: The minimum CPU platform. See
      https://cloud.google.com/compute/docs/instances/specify-min-cpu-
      platform. Not yet implemented.
    provisioningModel: The provisioning model.
  """

  class ProvisioningModelValueValuesEnum(_messages.Enum):
    r"""The provisioning model.

    Values:
      PROVISIONING_MODEL_UNSPECIFIED: Unspecified.
      STANDARD: Standard VM.
      SPOT: SPOT VM.
      PREEMPTIBLE: Preemptible VM (PVM). Above SPOT VM is the preferable model
        for preemptible VM instances: the old preemptible VM model (indicated
        by this field) is the older model, and has been migrated to use the
        SPOT model as the underlying technology. This old model will still be
        supported.
    """
    PROVISIONING_MODEL_UNSPECIFIED = 0
    STANDARD = 1
    SPOT = 2
    PREEMPTIBLE = 3

  accelerators = _messages.MessageField('Accelerator', 1, repeated=True)
  bootDisk = _messages.MessageField('Disk', 2)
  disks = _messages.MessageField('AttachedDisk', 3, repeated=True)
  machineType = _messages.StringField(4)
  minCpuPlatform = _messages.StringField(5)
  provisioningModel = _messages.EnumField('ProvisioningModelValueValuesEnum', 6)


class InstancePolicyOrTemplate(_messages.Message):
  r"""Either an InstancePolicy or an instance template.

  Fields:
    installGpuDrivers: Set this field true if users want Batch to help fetch
      drivers from a third party location and install them for GPUs specified
      in policy.accelerators or instance_template on their behalf. Default is
      false.
    instanceTemplate: Name of an instance template used to create VMs. Named
      the field as 'instance_template' instead of 'template' to avoid c++
      keyword conflict.
    policy: InstancePolicy.
  """

  installGpuDrivers = _messages.BooleanField(1)
  instanceTemplate = _messages.StringField(2)
  policy = _messages.MessageField('InstancePolicy', 3)


class InstanceStatus(_messages.Message):
  r"""VM instance status.

  Enums:
    ProvisioningModelValueValuesEnum: The VM instance provisioning model.

  Fields:
    bootDisk: The VM boot disk.
    machineType: The Compute Engine machine type.
    provisioningModel: The VM instance provisioning model.
    taskPack: The max number of tasks can be assigned to this instance type.
  """

  class ProvisioningModelValueValuesEnum(_messages.Enum):
    r"""The VM instance provisioning model.

    Values:
      PROVISIONING_MODEL_UNSPECIFIED: Unspecified.
      STANDARD: Standard VM.
      SPOT: SPOT VM.
      PREEMPTIBLE: Preemptible VM (PVM). Above SPOT VM is the preferable model
        for preemptible VM instances: the old preemptible VM model (indicated
        by this field) is the older model, and has been migrated to use the
        SPOT model as the underlying technology. This old model will still be
        supported.
    """
    PROVISIONING_MODEL_UNSPECIFIED = 0
    STANDARD = 1
    SPOT = 2
    PREEMPTIBLE = 3

  bootDisk = _messages.MessageField('Disk', 1)
  machineType = _messages.StringField(2)
  provisioningModel = _messages.EnumField('ProvisioningModelValueValuesEnum', 3)
  taskPack = _messages.IntegerField(4)


class Job(_messages.Message):
  r"""The Cloud Batch Job description.

  Messages:
    LabelsValue: Labels for the Job. Labels could be user provided or system
      generated. For example, "labels": { "department": "finance",
      "environment": "test" } You can assign up to 64 labels. [Google Compute
      Engine label
      restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) apply. Label names that start with "goog-" or
      "google-" are reserved.

  Fields:
    allocationPolicy: Compute resource allocation for all TaskGroups in the
      Job.
    createTime: Output only. When the Job was created.
    labels: Labels for the Job. Labels could be user provided or system
      generated. For example, "labels": { "department": "finance",
      "environment": "test" } You can assign up to 64 labels. [Google Compute
      Engine label
      restrictions](https://cloud.google.com/compute/docs/labeling-
      resources#restrictions) apply. Label names that start with "goog-" or
      "google-" are reserved.
    logsPolicy: Log preservation policy for the Job.
    name: Output only. Job name. For example: "projects/123456/locations/us-
      central1/jobs/job01".
    notifications: Notification configurations.
    priority: Priority of the Job. The valid value range is [0, 100). Default
      value is 0. Higher value indicates higher priority. A job with higher
      priority value is more likely to run earlier if all other requirements
      are satisfied.
    status: Output only. Job status. It is read only for users.
    taskGroups: Required. TaskGroups in the Job. Only one TaskGroup is
      supported now.
    uid: Output only. A system generated unique ID (in UUID4 format) for the
      Job.
    updateTime: Output only. The last time the Job was updated.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Labels for the Job. Labels could be user provided or system generated.
    For example, "labels": { "department": "finance", "environment": "test" }
    You can assign up to 64 labels. [Google Compute Engine label
    restrictions](https://cloud.google.com/compute/docs/labeling-
    resources#restrictions) apply. Label names that start with "goog-" or
    "google-" are reserved.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  allocationPolicy = _messages.MessageField('AllocationPolicy', 1)
  createTime = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  logsPolicy = _messages.MessageField('LogsPolicy', 4)
  name = _messages.StringField(5)
  notifications = _messages.MessageField('JobNotification', 6, repeated=True)
  priority = _messages.IntegerField(7)
  status = _messages.MessageField('JobStatus', 8)
  taskGroups = _messages.MessageField('TaskGroup', 9, repeated=True)
  uid = _messages.StringField(10)
  updateTime = _messages.StringField(11)


class JobNotification(_messages.Message):
  r"""Notification configurations.

  Fields:
    message: The attribute requirements of messages to be sent to this Pub/Sub
      topic. Without this field, no message will be sent.
    pubsubTopic: The Pub/Sub topic where notifications like the job state
      changes will be published. This topic exist in the same project as the
      job and billings will be charged to this project. If not specified, no
      Pub/Sub messages will be sent. Topic format:
      `projects/{project}/topics/{topic}`.
  """

  message = _messages.MessageField('Message', 1)
  pubsubTopic = _messages.StringField(2)


class JobStatus(_messages.Message):
  r"""Job status.

  Enums:
    StateValueValuesEnum: Job state

  Messages:
    TaskGroupsValue: Aggregated task status for each TaskGroup in the Job. The
      map key is TaskGroup ID.

  Fields:
    runDuration: The duration of time that the Job spent in status RUNNING.
    state: Job state
    statusEvents: Job status events
    taskGroups: Aggregated task status for each TaskGroup in the Job. The map
      key is TaskGroup ID.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Job state

    Values:
      STATE_UNSPECIFIED: <no description>
      QUEUED: Job is admitted (validated and persisted) and waiting for
        resources.
      SCHEDULED: Job is scheduled to run as soon as resource allocation is
        ready. The resource allocation may happen at a later time but with a
        high chance to succeed.
      RUNNING: Resource allocation has been successful. At least one Task in
        the Job is RUNNING.
      SUCCEEDED: All Tasks in the Job have finished successfully.
      FAILED: At least one Task in the Job has failed.
      DELETION_IN_PROGRESS: The Job will be deleted, but has not been deleted
        yet. Typically this is because resources used by the Job are still
        being cleaned up.
    """
    STATE_UNSPECIFIED = 0
    QUEUED = 1
    SCHEDULED = 2
    RUNNING = 3
    SUCCEEDED = 4
    FAILED = 5
    DELETION_IN_PROGRESS = 6

  @encoding.MapUnrecognizedFields('additionalProperties')
  class TaskGroupsValue(_messages.Message):
    r"""Aggregated task status for each TaskGroup in the Job. The map key is
    TaskGroup ID.

    Messages:
      AdditionalProperty: An additional property for a TaskGroupsValue object.

    Fields:
      additionalProperties: Additional properties of type TaskGroupsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a TaskGroupsValue object.

      Fields:
        key: Name of the additional property.
        value: A TaskGroupStatus attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('TaskGroupStatus', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  runDuration = _messages.StringField(1)
  state = _messages.EnumField('StateValueValuesEnum', 2)
  statusEvents = _messages.MessageField('StatusEvent', 3, repeated=True)
  taskGroups = _messages.MessageField('TaskGroupsValue', 4)


class KMSEnvMap(_messages.Message):
  r"""A KMSEnvMap object.

  Fields:
    cipherText: The value of the cipherText response from the `encrypt`
      method.
    keyName: The name of the KMS key that will be used to decrypt the cipher
      text.
  """

  cipherText = _messages.StringField(1)
  keyName = _messages.StringField(2)


class LifecyclePolicy(_messages.Message):
  r"""LifecyclePolicy describes how to deal with task failures based on
  different conditions.

  Enums:
    ActionValueValuesEnum: Action to execute when ActionCondition is true.
      When RETRY_TASK is specified, we will retry failed tasks if we notice
      any exit code match and fail tasks if no match is found. Likewise, when
      FAIL_TASK is specified, we will fail tasks if we notice any exit code
      match and retry tasks if no match is found.

  Fields:
    action: Action to execute when ActionCondition is true. When RETRY_TASK is
      specified, we will retry failed tasks if we notice any exit code match
      and fail tasks if no match is found. Likewise, when FAIL_TASK is
      specified, we will fail tasks if we notice any exit code match and retry
      tasks if no match is found.
    actionCondition: Conditions that decide why a task failure is dealt with a
      specific action.
  """

  class ActionValueValuesEnum(_messages.Enum):
    r"""Action to execute when ActionCondition is true. When RETRY_TASK is
    specified, we will retry failed tasks if we notice any exit code match and
    fail tasks if no match is found. Likewise, when FAIL_TASK is specified, we
    will fail tasks if we notice any exit code match and retry tasks if no
    match is found.

    Values:
      ACTION_UNSPECIFIED: Action unspecified.
      RETRY_TASK: Action that tasks in the group will be scheduled to re-
        execute.
      FAIL_TASK: Action that tasks in the group will be stopped immediately.
    """
    ACTION_UNSPECIFIED = 0
    RETRY_TASK = 1
    FAIL_TASK = 2

  action = _messages.EnumField('ActionValueValuesEnum', 1)
  actionCondition = _messages.MessageField('ActionCondition', 2)


class ListJobsResponse(_messages.Message):
  r"""ListJob Response.

  Fields:
    jobs: Jobs.
    nextPageToken: Next page token.
    unreachable: Locations that could not be reached.
  """

  jobs = _messages.MessageField('Job', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)


class ListTasksResponse(_messages.Message):
  r"""ListTasks Response.

  Fields:
    nextPageToken: Next page token.
    tasks: Tasks.
    unreachable: Locations that could not be reached.
  """

  nextPageToken = _messages.StringField(1)
  tasks = _messages.MessageField('Task', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents Google Cloud Platform location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LocationPolicy(_messages.Message):
  r"""A LocationPolicy object.

  Fields:
    allowedLocations: A list of allowed location names represented by internal
      URLs. Each location can be a region or a zone. Only one region or
      multiple zones in one region is supported now. For example,
      ["regions/us-central1"] allow VMs in any zones in region us-central1.
      ["zones/us-central1-a", "zones/us-central1-c"] only allow VMs in zones
      us-central1-a and us-central1-c. All locations end up in different
      regions would cause errors. For example, ["regions/us-central1",
      "zones/us-central1-a", "zones/us-central1-b", "zones/us-west1-a"]
      contains 2 regions "us-central1" and "us-west1". An error is expected in
      this case.
  """

  allowedLocations = _messages.StringField(1, repeated=True)


class LogsPolicy(_messages.Message):
  r"""LogsPolicy describes how outputs from a Job's Tasks (stdout/stderr) will
  be preserved.

  Enums:
    DestinationValueValuesEnum: Where logs should be saved.

  Fields:
    destination: Where logs should be saved.
    logsPath: The path to which logs are saved when the destination = PATH.
      This can be a local file path on the VM, or under the mount point of a
      Persistent Disk or Filestore, or a Cloud Storage path.
  """

  class DestinationValueValuesEnum(_messages.Enum):
    r"""Where logs should be saved.

    Values:
      DESTINATION_UNSPECIFIED: Logs are not preserved.
      CLOUD_LOGGING: Logs are streamed to Cloud Logging.
      PATH: Logs are saved to a file path.
    """
    DESTINATION_UNSPECIFIED = 0
    CLOUD_LOGGING = 1
    PATH = 2

  destination = _messages.EnumField('DestinationValueValuesEnum', 1)
  logsPath = _messages.StringField(2)


class Message(_messages.Message):
  r"""Message details. Describe the attribute that a message should have.
  Without specified message attributes, no message will be sent by default.

  Enums:
    NewJobStateValueValuesEnum: The new job state.
    NewTaskStateValueValuesEnum: The new task state.
    TypeValueValuesEnum: The message type.

  Fields:
    newJobState: The new job state.
    newTaskState: The new task state.
    type: The message type.
  """

  class NewJobStateValueValuesEnum(_messages.Enum):
    r"""The new job state.

    Values:
      STATE_UNSPECIFIED: <no description>
      QUEUED: Job is admitted (validated and persisted) and waiting for
        resources.
      SCHEDULED: Job is scheduled to run as soon as resource allocation is
        ready. The resource allocation may happen at a later time but with a
        high chance to succeed.
      RUNNING: Resource allocation has been successful. At least one Task in
        the Job is RUNNING.
      SUCCEEDED: All Tasks in the Job have finished successfully.
      FAILED: At least one Task in the Job has failed.
      DELETION_IN_PROGRESS: The Job will be deleted, but has not been deleted
        yet. Typically this is because resources used by the Job are still
        being cleaned up.
    """
    STATE_UNSPECIFIED = 0
    QUEUED = 1
    SCHEDULED = 2
    RUNNING = 3
    SUCCEEDED = 4
    FAILED = 5
    DELETION_IN_PROGRESS = 6

  class NewTaskStateValueValuesEnum(_messages.Enum):
    r"""The new task state.

    Values:
      STATE_UNSPECIFIED: unknown state
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5

  class TypeValueValuesEnum(_messages.Enum):
    r"""The message type.

    Values:
      TYPE_UNSPECIFIED: Unspecified.
      JOB_STATE_CHANGED: Notify users that the job state has changed.
      TASK_STATE_CHANGED: Notify users that the task state has changed.
    """
    TYPE_UNSPECIFIED = 0
    JOB_STATE_CHANGED = 1
    TASK_STATE_CHANGED = 2

  newJobState = _messages.EnumField('NewJobStateValueValuesEnum', 1)
  newTaskState = _messages.EnumField('NewTaskStateValueValuesEnum', 2)
  type = _messages.EnumField('TypeValueValuesEnum', 3)


class NFS(_messages.Message):
  r"""Represents an NFS volume.

  Fields:
    remotePath: Remote source path exported from the NFS, e.g., "/share".
    server: The IP address of the NFS.
  """

  remotePath = _messages.StringField(1)
  server = _messages.StringField(2)


class NetworkInterface(_messages.Message):
  r"""A network interface.

  Fields:
    network: The URL of an existing network resource. You can specify the
      network as a full or partial URL. For example, the following are all
      valid URLs: * https://www.googleapis.com/compute/v1/projects/{project}/g
      lobal/networks/{network} * projects/{project}/global/networks/{network}
      * global/networks/{network}
    noExternalIpAddress: Default is false (with an external IP address).
      Required if no external public IP address is attached to the VM. If no
      external public IP address, additional configuration is required to
      allow the VM to access Google Services. See
      https://cloud.google.com/vpc/docs/configure-private-google-access and
      https://cloud.google.com/nat/docs/gce-example#create-nat for more
      information.
    subnetwork: The URL of an existing subnetwork resource in the network. You
      can specify the subnetwork as a full or partial URL. For example, the
      following are all valid URLs: * https://www.googleapis.com/compute/v1/pr
      ojects/{project}/regions/{region}/subnetworks/{subnetwork} *
      projects/{project}/regions/{region}/subnetworks/{subnetwork} *
      regions/{region}/subnetworks/{subnetwork}
  """

  network = _messages.StringField(1)
  noExternalIpAddress = _messages.BooleanField(2)
  subnetwork = _messages.StringField(3)


class NetworkPolicy(_messages.Message):
  r"""NetworkPolicy describes VM instance network configurations.

  Fields:
    networkInterfaces: Network configurations.
  """

  networkInterfaces = _messages.MessageField('NetworkInterface', 1, repeated=True)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have
      successfully been cancelled have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Runnable(_messages.Message):
  r"""Runnable describes instructions for executing a specific script or
  container as part of a Task.

  Fields:
    alwaysRun: By default, after a Runnable fails, no further Runnable are
      executed. This flag indicates that this Runnable must be run even if the
      Task has already failed. This is useful for Runnables that copy output
      files off of the VM or for debugging. The always_run flag does not
      override the Task's overall max_run_duration. If the max_run_duration
      has expired then no further Runnables will execute, not even always_run
      Runnables.
    background: This flag allows a Runnable to continue running in the
      background while the Task executes subsequent Runnables. This is useful
      to provide services to other Runnables (or to provide debugging support
      tools like SSH servers).
    barrier: Barrier runnable.
    container: Container runnable.
    environment: Environment variables for this Runnable (overrides variables
      set for the whole Task or TaskGroup).
    ignoreExitStatus: Normally, a non-zero exit status causes the Task to
      fail. This flag allows execution of other Runnables to continue instead.
    script: Script runnable.
    timeout: Timeout for this Runnable.
  """

  alwaysRun = _messages.BooleanField(1)
  background = _messages.BooleanField(2)
  barrier = _messages.MessageField('Barrier', 3)
  container = _messages.MessageField('Container', 4)
  environment = _messages.MessageField('Environment', 5)
  ignoreExitStatus = _messages.BooleanField(6)
  script = _messages.MessageField('Script', 7)
  timeout = _messages.StringField(8)


class Script(_messages.Message):
  r"""Script runnable.

  Fields:
    path: Script file path on the host VM. To specify an interpreter, please
      add a `#!`(also known as [shebang
      line](https://en.wikipedia.org/wiki/Shebang_(Unix))) as the first line
      of the file.(For example, to execute the script using bash,
      `#!/bin/bash` should be the first line of the file. To execute the
      script using`Python3`, `#!/usr/bin/env python3` should be the first line
      of the file.) Otherwise, the file will by default be excuted by
      `/bin/sh`.
    text: Shell script text. To specify an interpreter, please add a `#!\n` at
      the beginning of the text.(For example, to execute the script using
      bash, `#!/bin/bash\n` should be added. To execute the script
      using`Python3`, `#!/usr/bin/env python3\n` should be added.) Otherwise,
      the script will by default be excuted by `/bin/sh`.
  """

  path = _messages.StringField(1)
  text = _messages.StringField(2)


class ServiceAccount(_messages.Message):
  r"""Carries information about a Google Cloud service account.

  Fields:
    email: Email address of the service account. If not specified, the default
      Compute Engine service account for the project will be used. If instance
      template is being used, the service account has to be specified in the
      instance template and it has to match the email field here.
    scopes: List of scopes to be enabled for this service account on the VM,
      in addition to the cloud-platform API scope that will be added by
      default.
  """

  email = _messages.StringField(1)
  scopes = _messages.StringField(2, repeated=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class StatusEvent(_messages.Message):
  r"""Status event

  Enums:
    TaskStateValueValuesEnum: Task State

  Fields:
    description: Description of the event.
    eventTime: The time this event occurred.
    taskExecution: Task Execution
    taskState: Task State
    type: Type of the event.
  """

  class TaskStateValueValuesEnum(_messages.Enum):
    r"""Task State

    Values:
      STATE_UNSPECIFIED: unknown state
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5

  description = _messages.StringField(1)
  eventTime = _messages.StringField(2)
  taskExecution = _messages.MessageField('TaskExecution', 3)
  taskState = _messages.EnumField('TaskStateValueValuesEnum', 4)
  type = _messages.StringField(5)


class Task(_messages.Message):
  r"""A Cloud Batch task.

  Fields:
    name: Task name. The name is generated from the parent TaskGroup name and
      'id' field. For example: "projects/123456/locations/us-
      west1/jobs/job01/taskGroups/group01/tasks/task01".
    status: Task Status.
  """

  name = _messages.StringField(1)
  status = _messages.MessageField('TaskStatus', 2)


class TaskExecution(_messages.Message):
  r"""This Task Execution field includes detail information for task execution
  procedures, based on StatusEvent types.

  Fields:
    exitCode: When task is completed as the status of FAILED or SUCCEEDED,
      exit code is for one task execution result, default is 0 as success.
  """

  exitCode = _messages.IntegerField(1, variant=_messages.Variant.INT32)


class TaskGroup(_messages.Message):
  r"""A TaskGroup contains one or multiple Tasks that share the same Runnable
  but with different runtime parameters.

  Fields:
    name: Output only. TaskGroup name. The system generates this field based
      on parent Job name. For example: "projects/123456/locations/us-
      west1/jobs/job01/taskGroups/group01".
    parallelism: Max number of tasks that can run in parallel. Default to
      min(task_count, 1000).
    permissiveSsh: When true, Batch will configure SSH to allow passwordless
      login between VMs running the Batch tasks in the same TaskGroup.
    requireHostsFile: When true, Batch will populate a file with a list of all
      VMs assigned to the TaskGroup and set the BATCH_HOSTS_FILE environment
      variable to the path of that file. Defaults to false.
    taskCount: Number of Tasks in the TaskGroup. Default is 1.
    taskCountPerNode: Max number of tasks that can be run on a VM at the same
      time. If not specified, the system will decide a value based on
      available compute resources on a VM and task requirements.
    taskEnvironments: An array of environment variable mappings, which are
      passed to Tasks with matching indices. If task_environments is used then
      task_count should not be specified in the request (and will be ignored).
      Task count will be the length of task_environments. Tasks get a
      BATCH_TASK_INDEX and BATCH_TASK_COUNT environment variable, in addition
      to any environment variables set in task_environments, specifying the
      number of Tasks in the Task's parent TaskGroup, and the specific Task's
      index in the TaskGroup (0 through BATCH_TASK_COUNT - 1).
      task_environments supports up to 200 entries.
    taskSpec: Required. Tasks in the group share the same task spec.
  """

  name = _messages.StringField(1)
  parallelism = _messages.IntegerField(2)
  permissiveSsh = _messages.BooleanField(3)
  requireHostsFile = _messages.BooleanField(4)
  taskCount = _messages.IntegerField(5)
  taskCountPerNode = _messages.IntegerField(6)
  taskEnvironments = _messages.MessageField('Environment', 7, repeated=True)
  taskSpec = _messages.MessageField('TaskSpec', 8)


class TaskGroupStatus(_messages.Message):
  r"""Aggregated task status for a TaskGroup.

  Messages:
    CountsValue: Count of task in each state in the TaskGroup. The map key is
      task state name.

  Fields:
    counts: Count of task in each state in the TaskGroup. The map key is task
      state name.
    instances: Status of instances allocated for the TaskGroup.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class CountsValue(_messages.Message):
    r"""Count of task in each state in the TaskGroup. The map key is task
    state name.

    Messages:
      AdditionalProperty: An additional property for a CountsValue object.

    Fields:
      additionalProperties: Additional properties of type CountsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a CountsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.IntegerField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  counts = _messages.MessageField('CountsValue', 1)
  instances = _messages.MessageField('InstanceStatus', 2, repeated=True)


class TaskSpec(_messages.Message):
  r"""Spec of a task

  Messages:
    EnvironmentsValue: Deprecated: please use environment(non-plural) instead.

  Fields:
    computeResource: ComputeResource requirements.
    environment: Environment variables to set before running the Task.
    environments: Deprecated: please use environment(non-plural) instead.
    lifecyclePolicies: Lifecycle management schema when any task in a task
      group is failed. Currently we only support one lifecycle policy. When
      the lifecycle policy condition is met, the action in the policy will
      execute. If task execution result does not meet with the defined
      lifecycle policy, we consider it as the default policy. Default policy
      means if the exit code is 0, exit task. If task ends with non-zero exit
      code, retry the task with max_retry_count.
    maxRetryCount: Maximum number of retries on failures. The default, 0,
      which means never retry. The valid value range is [0, 10].
    maxRunDuration: Maximum duration the task should run. The task will be
      killed and marked as FAILED if over this limit.
    runnables: The sequence of scripts or containers to run for this Task.
      Each Task using this TaskSpec executes its list of runnables in order.
      The Task succeeds if all of its runnables either exit with a zero status
      or any that exit with a non-zero status have the ignore_exit_status
      flag. Background runnables are killed automatically (if they have not
      already exited) a short time after all foreground runnables have
      completed. Even though this is likely to result in a non-zero exit
      status for the background runnable, these automatic kills are not
      treated as Task failures.
    volumes: Volumes to mount before running Tasks using this TaskSpec.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EnvironmentsValue(_messages.Message):
    r"""Deprecated: please use environment(non-plural) instead.

    Messages:
      AdditionalProperty: An additional property for a EnvironmentsValue
        object.

    Fields:
      additionalProperties: Additional properties of type EnvironmentsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EnvironmentsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  computeResource = _messages.MessageField('ComputeResource', 1)
  environment = _messages.MessageField('Environment', 2)
  environments = _messages.MessageField('EnvironmentsValue', 3)
  lifecyclePolicies = _messages.MessageField('LifecyclePolicy', 4, repeated=True)
  maxRetryCount = _messages.IntegerField(5, variant=_messages.Variant.INT32)
  maxRunDuration = _messages.StringField(6)
  runnables = _messages.MessageField('Runnable', 7, repeated=True)
  volumes = _messages.MessageField('Volume', 8, repeated=True)


class TaskStatus(_messages.Message):
  r"""Status of a task

  Enums:
    StateValueValuesEnum: Task state

  Fields:
    state: Task state
    statusEvents: Detailed info about why the state is reached.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Task state

    Values:
      STATE_UNSPECIFIED: unknown state
      PENDING: The Task is created and waiting for resources.
      ASSIGNED: The Task is assigned to at least one VM.
      RUNNING: The Task is running.
      FAILED: The Task has failed.
      SUCCEEDED: The Task has succeeded.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    ASSIGNED = 2
    RUNNING = 3
    FAILED = 4
    SUCCEEDED = 5

  state = _messages.EnumField('StateValueValuesEnum', 1)
  statusEvents = _messages.MessageField('StatusEvent', 2, repeated=True)


class Volume(_messages.Message):
  r"""Volume describes a volume and parameters for it to be mounted to a VM.

  Fields:
    deviceName: Device name of an attached disk volume, which should align
      with a device_name specified by
      job.allocation_policy.instances[0].policy.disks[i].device_name or
      defined by the given instance template in
      job.allocation_policy.instances[0].instance_template.
    gcs: A Google Cloud Storage (GCS) volume.
    mountOptions: For Google Cloud Storage (GCS), mount options are the
      options supported by the gcsfuse tool
      (https://github.com/GoogleCloudPlatform/gcsfuse). For existing
      persistent disks, mount options provided by the mount command
      (https://man7.org/linux/man-pages/man8/mount.8.html) except writing are
      supported. This is due to restrictions of multi-writer mode
      (https://cloud.google.com/compute/docs/disks/sharing-disks-between-vms).
      For other attached disks and Network File System (NFS), mount options
      are these supported by the mount command (https://man7.org/linux/man-
      pages/man8/mount.8.html).
    mountPath: The mount path for the volume, e.g. /mnt/disks/share.
    nfs: A Network File System (NFS) volume. For example, a Filestore file
      share.
  """

  deviceName = _messages.StringField(1)
  gcs = _messages.MessageField('GCS', 2)
  mountOptions = _messages.StringField(3, repeated=True)
  mountPath = _messages.StringField(4)
  nfs = _messages.MessageField('NFS', 5)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Bare Metal Solution client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import collections
import io
import re

from apitools.base.py import exceptions as apitools_exceptions
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import exceptions as apilib_exceptions
from googlecloudsdk.command_lib.bms import util
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.resource import resource_printer


import six

_DEFAULT_API_VERSION = 'v2'
_V1_API_VERSION = 'v1'
_GLOBAL_REGION = 'global'
_REGIONAL_IAM_REGEX = re.compile(
    "PERMISSION_DENIED: Permission (.+) denied on 'projects/(.+?)/.*")

IpRangeReservation = collections.namedtuple(
    'IpRangeReservation', ['start_address', 'end_address', 'note'])


def _ParseError(error):
  """Returns a best-effort error message created from an API client error."""
  if isinstance(error, apitools_exceptions.HttpError):
    parsed_error = apilib_exceptions.HttpException(error,
                                                   error_format='{message}')
    error_message = parsed_error.message
  else:
    error_message = six.text_type(error)
  return error_message


def _CollapseRegionalIAMErrors(errors):
  """If all errors are PERMISSION_DENIEDs, use a single global error instead."""
  # TODO(b/198857865): Remove this hack once the `global` region fix is in
  if errors:
    matches = [_REGIONAL_IAM_REGEX.match(e) for e in errors]
    if (all(match is not None for match in matches)
        and len(set(match.group(1) for match in matches)) == 1):
      errors = ['PERMISSION_DENIED: Permission %s denied on projects/%s' %
                (matches[0].group(1), matches[0].group(2))]
  return errors


class BmsClient(object):
  """Cloud Bare Metal Solution client."""

  def __init__(self, api_version=_DEFAULT_API_VERSION):
    self._client = apis.GetClientInstance('baremetalsolution', api_version)
    self._v1_client = apis.GetClientInstance('baremetalsolution',
                                             _V1_API_VERSION)
    self._messages = apis.GetMessagesModule('baremetalsolution', api_version)
    self.instances_service = self._client.projects_locations_instances
    self.volumes_service = self._client.projects_locations_volumes
    self.snapshot_schedule_policies_service = self._client.projects_locations_snapshotSchedulePolicies
    self.snapshots_service = self._client.projects_locations_volumes_snapshots
    self.networks_service = self._client.projects_locations_networks
    self.locations_service = self._client.projects_locations
    self.luns_service = self._client.projects_locations_volumes_luns
    self.nfs_shares_service = self._client.projects_locations_nfsShares
    self.ssh_keys_service = self._client.projects_locations_sshKeys
    self.operation_service = self._client.projects_locations_operations
    self.nfs_mount_permissions_str_to_message = {
        'READ_ONLY':
            self.messages.AllowedClient.MountPermissionsValueValuesEnum.READ,
        'READ_WRITE':
            self.messages.AllowedClient.MountPermissionsValueValuesEnum
            .READ_WRITE,
    }
    self.nfs_storage_type_str_to_message = {
        'SSD': self.messages.NfsShare.StorageTypeValueValuesEnum.SSD,
        'HDD': self.messages.NfsShare.StorageTypeValueValuesEnum.HDD,
    }

  @property
  def client(self):
    return self._client

  @property
  def messages(self):
    return self._messages

  def GetInstance(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsInstancesGetRequest(
        name=resource.RelativeName())
    return self.instances_service.Get(request)

  def AggregateYieldFromList(self,
                             service,
                             project_resource,
                             request_class,
                             resource,
                             global_params=None,
                             limit=None,
                             method='List',
                             predicate=None,
                             skip_global_region=True,
                             allow_partial_server_failure=True):
    """Make a series of List requests, across locations in a project.

    Args:
      service: apitools_base.BaseApiService, A service with a .List() method.
      project_resource: str, The resource name of the project.
      request_class: class, The class type of the List RPC request.
      resource: string, The name (in plural) of the resource type.
      global_params: protorpc.messages.Message, The global query parameters to
        provide when calling the given method.
      limit: int, The maximum number of records to yield. None if all available
        records should be yielded.
      method: str, The name of the method used to fetch resources.
      predicate: lambda, A function that returns true for items to be yielded.
      skip_global_region: bool, True if global region must be filtered out while
      iterating over regions
      allow_partial_server_failure: bool, if True don't fail and only print a
        warning if some requests fail as long as at least one succeeds. If
        False, fail the complete command if at least one request fails.

    Yields:
      protorpc.message.Message, The resources listed by the service.

    """
    response_count = 0
    errors = []
    for location in self.ListLocations(project_resource):
      # TODO(b/198857865): Global region will be used when it is ready.
      location_name = location.name.split('/')[-1]
      if skip_global_region and location_name == _GLOBAL_REGION:
        continue
      request = request_class(parent=location.name)
      try:
        response = getattr(service, method)(
            request, global_params=global_params)
        response_count += 1
      except Exception as e:  # pylint: disable=broad-except
        errors.append(_ParseError(e))
        continue
      items = getattr(response, resource)
      if predicate:
        items = list(filter(predicate, items))
      for item in items:
        yield item
        if limit is None:
          continue
        limit -= 1
        if not limit:
          break

    if errors:
      # If the command allows partial server errors, instead of raising an
      # exception to show something went wrong, we show a warning message that
      # contains the error messages instead.
      buf = io.StringIO()
      fmt = ('list[title="Some requests did not succeed.",'
             'always-display-title]')
      if allow_partial_server_failure and response_count > 0:
        resource_printer.Print(sorted(set(errors)), fmt, out=buf)
        log.warning(buf.getvalue())
      else:
        # If all requests failed, clean them up if they're duplicated IAM errors
        collapsed_errors = _CollapseRegionalIAMErrors(errors)
        resource_printer.Print(sorted(set(collapsed_errors)), fmt, out=buf)
        raise exceptions.Error(buf.getvalue())

  def ListLocations(self,
                    project_resource,
                    limit=None,
                    page_size=None):
    request = self.messages.BaremetalsolutionProjectsLocationsListRequest(
        name='projects/' + project_resource)
    return list_pager.YieldFromList(
        self.locations_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='locations')

  def ListInstances(self, location_resource, limit=None, page_size=None):
    location = location_resource.RelativeName()
    request = self.messages.BaremetalsolutionProjectsLocationsInstancesListRequest(
        parent=location)
    return list_pager.YieldFromList(
        self.instances_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='instances')

  def AggregateListInstances(self, project_resource, limit=None):
    return self.AggregateYieldFromList(
        self.instances_service,
        project_resource,
        self.messages.BaremetalsolutionProjectsLocationsInstancesListRequest,
        'instances',
        limit=limit)

  def UpdateInstance(self, instance_resource, labels, os_image,
                     enable_hyperthreading):
    """Update an existing instance resource."""
    updated_fields = []
    if labels is not None:
      updated_fields.append('labels')
    if os_image is not None:
      updated_fields.append('os_image')
    if enable_hyperthreading is not None:
      updated_fields.append('hyperthreading_enabled')

    instance_msg = self.messages.Instance(
        name=instance_resource.RelativeName(), labels=labels, osImage=os_image,
        hyperthreadingEnabled=enable_hyperthreading)

    request = self.messages.BaremetalsolutionProjectsLocationsInstancesPatchRequest(
        name=instance_resource.RelativeName(),
        instance=instance_msg,
        updateMask=','.join(updated_fields))

    return self.instances_service.Patch(request)

  def ListSnapshotSchedulePolicies(self,
                                   project_resource,
                                   limit=None,
                                   page_size=None):
    parent = 'projects/%s/locations/global' % project_resource
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesListRequest(
            parent=parent))
    return list_pager.YieldFromList(
        self.snapshot_schedule_policies_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='snapshotSchedulePolicies')

  def GetSnapshotSchedulePolicy(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesGetRequest(
        name=resource.RelativeName())
    return self.snapshot_schedule_policies_service.Get(request)

  def CreateSnapshotSchedulePolicy(self,
                                   policy_resource,
                                   labels,
                                   description,
                                   schedules):
    """Sends request to create a new Snapshot Schedule Policy."""
    policy_id = policy_resource.Name()
    parent = policy_resource.Parent().RelativeName()
    schedule_msgs = self._ParseSnapshotSchedules(schedules)
    policy_msg = self.messages.SnapshotSchedulePolicy(
        description=description,
        schedules=schedule_msgs,
        labels=labels)
    request = self.messages.BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesCreateRequest(
        parent=parent,
        snapshotSchedulePolicyId=policy_id,
        snapshotSchedulePolicy=policy_msg)
    return self.snapshot_schedule_policies_service.Create(request)

  def UpdateSnapshotSchedulePolicy(self,
                                   policy_resource,
                                   description,
                                   labels,
                                   schedules):
    """Sends request to update an existing SnapshotSchedulePolicy."""
    updated_fields = []
    if description:
      updated_fields.append('description')
    if labels is not None:
      updated_fields.append('labels')
    schedule_msgs = self._ParseSnapshotSchedules(schedules)
    if schedule_msgs:
      updated_fields.append('schedules')

    update_mask = ','.join(updated_fields)
    policy_msg = self.messages.SnapshotSchedulePolicy(
        description=description,
        schedules=schedule_msgs,
        labels=labels)
    request = self.messages.BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesPatchRequest(
        name=policy_resource.RelativeName(),
        snapshotSchedulePolicy=policy_msg,
        updateMask=update_mask)
    return self.snapshot_schedule_policies_service.Patch(request)

  def _ParseSnapshotSchedules(self, schedules):
    """Parses schedule ArgDict dicts into a list of Schedule messages."""
    schedule_msgs = []
    if schedules:
      for schedule_arg in schedules:
        schedule_msgs.append(self.messages.Schedule(
            crontabSpec=schedule_arg['crontab_spec'],
            retentionCount=schedule_arg['retention_count'],
            prefix=schedule_arg['prefix']))
    return schedule_msgs

  def DeleteSnapshotSchedulePolicy(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsSnapshotSchedulePoliciesDeleteRequest(
        name=resource.RelativeName())
    return self.snapshot_schedule_policies_service.Delete(request)

  def ListVolumes(self,
                  location_resource,
                  limit=None,
                  page_size=None):
    location = location_resource.RelativeName()
    request = self.messages.BaremetalsolutionProjectsLocationsVolumesListRequest(
        parent=location)
    return list_pager.YieldFromList(
        self.volumes_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='volumes')

  def UpdateVolume(self,
                   volume_resource,
                   labels,
                   snapshot_schedule_policy_resource,
                   remove_snapshot_schedule_policy,
                   snapshot_auto_delete):
    """Update an existing volume resource."""
    updated_fields = []
    policy_name = None
    if snapshot_schedule_policy_resource:
      updated_fields.append('snapshotSchedulePolicy')
      policy_name = snapshot_schedule_policy_resource.RelativeName()
    elif remove_snapshot_schedule_policy:
      updated_fields.append('snapshotSchedulePolicy')
    if labels is not None:
      updated_fields.append('labels')
    if snapshot_auto_delete:
      updated_fields.append('snapshotAutoDeleteBehavior')

    volume_msg = self.messages.Volume(
        name=volume_resource.RelativeName(),
        snapshotAutoDeleteBehavior=snapshot_auto_delete,
        snapshotSchedulePolicy=policy_name,
        labels=labels)

    request = self.messages.BaremetalsolutionProjectsLocationsVolumesPatchRequest(
        name=volume_resource.RelativeName(),
        volume=volume_msg,
        updateMask=','.join(updated_fields))

    return self.volumes_service.Patch(request)

  def GetVolume(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsVolumesGetRequest(
        name=resource.RelativeName())
    return self.volumes_service.Get(request)

  def AggregateListVolumes(self, project_resource, limit=None):
    return self.AggregateYieldFromList(
        self.volumes_service,
        project_resource,
        self.messages.BaremetalsolutionProjectsLocationsVolumesListRequest,
        'volumes',
        limit=limit)

  def ListNetworks(self,
                   location_resource,
                   limit=None,
                   page_size=None):
    location = location_resource.RelativeName()
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsNetworksListRequest(
            parent=location))
    return list_pager.YieldFromList(
        self.networks_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='networks')

  def AggregateListNetworks(self, project_resource, limit=None):
    return self.AggregateYieldFromList(
        self.networks_service,
        project_resource,
        self.messages.BaremetalsolutionProjectsLocationsNetworksListRequest,
        'networks',
        limit=limit)

  def GetNetwork(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsNetworksGetRequest(
        name=resource.RelativeName())
    return self.networks_service.Get(request)

  def UpdateNetwork(self, network_resource, labels, ip_reservations):
    """Update an existing network resource."""
    updated_fields = []
    ip_reservations_messages = []
    if labels is not None:
      updated_fields.append('labels')
    if ip_reservations is not None:
      updated_fields.append('reservations')
      for ip_reservation in ip_reservations:
        ip_reservations_messages.append(self.messages.NetworkAddressReservation(
            startAddress=ip_reservation.start_address,
            endAddress=ip_reservation.end_address,
            note=ip_reservation.note,
            ))

    network_msg = self.messages.Network(
        name=network_resource.RelativeName(), labels=labels,
        reservations=ip_reservations_messages)

    request = self.messages.BaremetalsolutionProjectsLocationsNetworksPatchRequest(
        name=network_resource.RelativeName(),
        network=network_msg,
        updateMask=','.join(updated_fields))

    return self.networks_service.Patch(request)

  def IsClientNetwork(self, network):
    return network.type == self.messages.Network.TypeValueValuesEnum.CLIENT

  def IsPrivateNetwork(self, network):
    return network.type == self.messages.Network.TypeValueValuesEnum.PRIVATE

  def IsClientLogicalNetworkInterface(self, logical_network_interface):
    return logical_network_interface.networkType == self.messages.LogicalNetworkInterface.NetworkTypeValueValuesEnum.CLIENT

  def IsPrivateLogicalNetworkInterface(self, logical_network_interface):
    return logical_network_interface.networkType == self.messages.LogicalNetworkInterface.NetworkTypeValueValuesEnum.PRIVATE

  def ListLUNsForVolume(self, volume_resource, limit=None,
                        page_size=None):
    parent = volume_resource.RelativeName()
    request = (self.messages
               .BaremetalsolutionProjectsLocationsVolumesLunsListRequest(
                   parent=parent))
    return list_pager.YieldFromList(
        self.luns_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='luns')

  def GetLUN(self, resource):
    request = (self.messages
               .BaremetalsolutionProjectsLocationsVolumesLunsGetRequest(
                   name=resource.RelativeName()))
    return self.luns_service.Get(request)

  def ListSnapshotsForVolume(self,
                             volume_resource,
                             limit=None,
                             page_size=None):
    parent = volume_resource.RelativeName()
    request = (self.messages
               .BaremetalsolutionProjectsLocationsVolumesSnapshotsListRequest(
                   parent=parent))
    return list_pager.YieldFromList(
        self.snapshots_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='volumeSnapshots')

  def GetVolumeSnapshot(self, resource):
    request = (self.messages
               .BaremetalsolutionProjectsLocationsVolumesSnapshotsGetRequest(
                   name=resource.RelativeName()))
    return self.snapshots_service.Get(request)

  def CreateVolumeSnapshot(self, resource, name, description):
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsVolumesSnapshotsCreateRequest(
            parent=resource.RelativeName(),
            volumeSnapshot=self.messages.VolumeSnapshot(
                name=name, description=description)))
    return self.snapshots_service.Create(request)

  def DeleteVolumeSnapshot(self, resource):
    request = (self.messages
               .BaremetalsolutionProjectsLocationsVolumesSnapshotsDeleteRequest(
                   name=resource.RelativeName()))
    return self.snapshots_service.Delete(request)

  def RestoreVolumeSnapshot(self, snapshot_name):
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsVolumesSnapshotsRestoreVolumeSnapshotRequest(  # pylint: disable=line-too-long
            volumeSnapshot=snapshot_name))
    return self.snapshots_service.RestoreVolumeSnapshot(request)

  def GetNfsShare(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsNfsSharesGetRequest(
        name=resource.RelativeName())
    return self.nfs_shares_service.Get(request)

  def AggregateListNfsShares(self, project_resource, limit=None):
    return self.AggregateYieldFromList(
        self.nfs_shares_service,
        project_resource,
        self.messages.BaremetalsolutionProjectsLocationsNfsSharesListRequest,
        'nfsShares',
        limit=limit)

  def ListNfsShares(self,
                    location_resource,
                    limit=None,
                    page_size=None):
    location = location_resource.RelativeName()
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsNfsSharesListRequest(
            parent=location))
    return list_pager.YieldFromList(
        self.nfs_shares_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='nfsShares')

  def UpdateNfsShare(self, nfs_share_resource, labels, allowed_clients):
    """Update an existing nfs share resource."""
    updated_fields = []
    updated_allowed_clients = []
    if labels is not None:
      updated_fields.append('labels')
    if allowed_clients is not None:
      updated_fields.append('allowedClients')
      updated_allowed_clients = allowed_clients

    nfs_share_msg = self.messages.NfsShare(
        name=nfs_share_resource.RelativeName(), labels=labels,
        allowedClients=updated_allowed_clients)

    request = self.messages.BaremetalsolutionProjectsLocationsNfsSharesPatchRequest(
        name=nfs_share_resource.RelativeName(),
        nfsShare=nfs_share_msg,
        updateMask=','.join(updated_fields))

    return self.nfs_shares_service.Patch(request)

  def DeleteNfsShare(self, nfs_share_resource):
    """Delete an existing nfs share resource."""
    request = self.messages.BaremetalsolutionProjectsLocationsNfsSharesDeleteRequest(
        name=nfs_share_resource.RelativeName())
    return self.nfs_shares_service.Delete(request)

  def CreateNfsShare(self,
                     nfs_share_resource,
                     size_gib,
                     storage_type,
                     allowed_clients_dicts,
                     labels):
    """Create an NFS share resource."""
    allowed_clients = self.ParseAllowedClientsDicts(
        nfs_share_resource=nfs_share_resource,
        allowed_clients_dicts=allowed_clients_dicts)
    nfs_share_msg = self.messages.NfsShare(
        name=nfs_share_resource.RelativeName(),
        requestedSizeGib=size_gib,
        storageType=self.nfs_storage_type_str_to_message[storage_type],
        allowedClients=allowed_clients,
        labels=labels)
    request = self.messages.BaremetalsolutionProjectsLocationsNfsSharesCreateRequest(
        nfsShare=nfs_share_msg,
        parent=nfs_share_resource.Parent().RelativeName())
    return self.nfs_shares_service.Create(request)

  def ListSshKeys(self,
                  project_resource,
                  limit=None,
                  page_size=None):
    parent = 'projects/%s/locations/global' % project_resource
    request = (
        self.messages
        .BaremetalsolutionProjectsLocationsSshKeysListRequest(
            parent=parent))
    return list_pager.YieldFromList(
        self.ssh_keys_service,
        request,
        limit=limit,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        field='sshKeys')

  def CreateSshKey(self,
                   resource,
                   public_key):
    """Sends request to create an SSH key."""
    request = self.messages.BaremetalsolutionProjectsLocationsSshKeysCreateRequest(
        parent=resource.Parent().RelativeName(),
        sshKeyId=resource.Name(),
        sSHKey=self.messages.SSHKey(publicKey=public_key))
    return self.ssh_keys_service.Create(request)

  def DeleteSshKey(self, resource):
    request = self.messages.BaremetalsolutionProjectsLocationsSshKeysDeleteRequest(
        name=resource.RelativeName())
    return self.ssh_keys_service.Delete(request)

  def ParseAllowedClientsDicts(self, nfs_share_resource, allowed_clients_dicts):
    """Parses NFS share allowed client list of dicts."""
    allowed_clients = []
    for allowed_client in allowed_clients_dicts:
      mount_permissions = self.nfs_mount_permissions_str_to_message[
          allowed_client['mount-permissions']]
      network_full_name = util.NFSNetworkFullName(
          nfs_share_resource=nfs_share_resource,
          allowed_client_dict=allowed_client)
      allowed_clients.append(
          self.messages.AllowedClient(
              network=network_full_name,
              allowedClientsCidr=allowed_client['cidr'],
              mountPermissions=mount_permissions,
              allowDev=allowed_client['allow-dev'],
              allowSuid=allowed_client['allow-suid'],
              noRootSquash=not allowed_client['enable-root-squash'],
              )
          )
    return allowed_clients




"""Generated message classes for workloadcertificate version v1alpha1.

workloadcertificate.googleapis.com API.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'workloadcertificate'


class CaPoolsStatus(_messages.Message):
  r"""Status of CA pools in a region.

  Fields:
    caPools: The CA pool string has a relative resource path following the
      form "projects/{project_number}/locations/{location}/caPools/{ca_pool}".
  """

  caPools = _messages.StringField(1, repeated=True)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class GCPAttributeSelector(_messages.Message):
  r"""GCPAttributeSelector is an "oneof" field, which can be a Google service
  account, a resource identity, etc. If not specified, all workloads of the
  given type in the project are selected.

  Fields:
    serviceAccount: Selects workloads with the GCP service account. It matches
      the "email" claim.
  """

  serviceAccount = _messages.StringField(1)


class GCPWorkloadSelector(_messages.Message):
  r"""GCPWorkloadSelector selects a group of GCP workloads, like GCE or load
  balancer.

  Enums:
    TypeValueValuesEnum: Type pecifies the workload type. It is either
      indicated by the identity dataplane P4SA, or in the resource identity
      case, the resource service ("google.resource.service" claim). required.

  Fields:
    attribute: GCPAttributeSelector selects a group of workloads that match
      the attribute. optional.
    project: Project specifies the workloads' project in the format of "".
      required.
    type: Type pecifies the workload type. It is either indicated by the
      identity dataplane P4SA, or in the resource identity case, the resource
      service ("google.resource.service" claim). required.
  """

  class TypeValueValuesEnum(_messages.Enum):
    r"""Type pecifies the workload type. It is either indicated by the
    identity dataplane P4SA, or in the resource identity case, the resource
    service ("google.resource.service" claim). required.

    Values:
      WORKLOAD_TYPE_UNSPECIFIED: Default value. This value is unused and this
        field is invalid.
      GOOGLE_COMPUTE_ENGINE: <no description>
    """
    WORKLOAD_TYPE_UNSPECIFIED = 0
    GOOGLE_COMPUTE_ENGINE = 1

  attribute = _messages.MessageField('GCPAttributeSelector', 1)
  project = _messages.StringField(2)
  type = _messages.EnumField('TypeValueValuesEnum', 3)


class K8SWorkloadSelector(_messages.Message):
  r"""For kubernetes workloads, fleet_member_id is used as workload selector.

  Fields:
    fleetMemberId: Fleet membership ID (only the name part, not the full URI).
      The "project" and "location" of the membership is the same as the
      WorkloadRegistration. required.
  """

  fleetMemberId = _messages.StringField(1)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)


class ListWorkloadRegistrationsResponse(_messages.Message):
  r"""Response message for response to listing WorkloadRegistrations.

  Fields:
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
    workloadRegistrations: The list of WorkloadRegistration
  """

  nextPageToken = _messages.StringField(1)
  unreachable = _messages.StringField(2, repeated=True)
  workloadRegistrations = _messages.MessageField('WorkloadRegistration', 3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents Google Cloud Platform location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class RegistrationStatus(_messages.Message):
  r"""RegistrationStatus describes the certificate provisioning status of a
  workload registration resource.

  Enums:
    StateValueValuesEnum: state represents the current state of registration.

  Fields:
    error: error describes the details when a certificate provisioning process
      fails.
    state: state represents the current state of registration.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""state represents the current state of registration.

    Values:
      REGISTRATION_STATE_UNSPECIFIED: REGISTRATION_STATE_UNSPECIFIED is the
        default value.
      REGISTRATION_STATE_READY: REGISTRATION_STATE_READY indicates that the
        registration is ready.
      REGISTRATION_STATE_IN_PROGRESS: REGISTRATION_STATE_IN_PROGRESS indicates
        that the registration is in progress.
      REGISTRATION_STATE_INTERNAL_ERROR: REGISTRATION_STATE_INTERNAL_ERROR
        indicates that the registration fails because of some internal errors.
    """
    REGISTRATION_STATE_UNSPECIFIED = 0
    REGISTRATION_STATE_READY = 1
    REGISTRATION_STATE_IN_PROGRESS = 2
    REGISTRATION_STATE_INTERNAL_ERROR = 3

  error = _messages.StringField(1)
  state = _messages.EnumField('StateValueValuesEnum', 2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class WorkloadCertificateFeature(_messages.Message):
  r"""Represents the Managed Workload Certificate feature. This is a singleton
  resource of a project that contains the mode of the feature, trust-domain-
  level configurations, and etc. go/workloadcert-enablement-api.

  Fields:
    defaultSpec: Optional. Workload certificate feature spec for the default
      project level trust domain (i.e. `.svc.id.goog`). If left unset, the
      feature is disabled.
    defaultStatus: Output only. The current WorkloadCertificate feature status
      of the default project level trust domain.
    name: Output only. Name of the resource. Format:
      `projects/*/locations/global/workloadCertificateFeature`.
  """

  defaultSpec = _messages.MessageField('WorkloadCertificateFeatureSpec', 1)
  defaultStatus = _messages.MessageField('WorkloadCertificateFeatureStatus', 2)
  name = _messages.StringField(3)


class WorkloadCertificateFeatureSpec(_messages.Message):
  r"""Spec for the workload certificate feature.

  Enums:
    ModeValueValuesEnum: The mode for the workload certificate feature.

  Fields:
    mode: The mode for the workload certificate feature.
  """

  class ModeValueValuesEnum(_messages.Enum):
    r"""The mode for the workload certificate feature.

    Values:
      MODE_UNSPECIFIED: Workload certificate feature is disabled.
      MODE_DISABLED: Workload certificate feature is disabled.
      MODE_ENABLED_WITH_MANAGED_CA: Workload certificate feature is enabled,
        and the entire certificate provisioning process is managed by Google,
        including the CA.
    """
    MODE_UNSPECIFIED = 0
    MODE_DISABLED = 1
    MODE_ENABLED_WITH_MANAGED_CA = 2

  mode = _messages.EnumField('ModeValueValuesEnum', 1)


class WorkloadCertificateFeatureStatus(_messages.Message):
  r"""Status of Workload Certificate feature at trust domain level.

  Enums:
    StateValueValuesEnum: Describes whether the Workload Certificate feature
      meets its spec.

  Messages:
    ManagedCaPoolsValue: A map from a region to the status of managed CA pools
      in that region.

  Fields:
    managedCaPools: A map from a region to the status of managed CA pools in
      that region.
    state: Describes whether the Workload Certificate feature meets its spec.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Describes whether the Workload Certificate feature meets its spec.

    Values:
      FEATURE_STATE_UNSPECIFIED: The feature status does not fully meet its
        spec at the moment but is trying to meet its spec.
      FEATURE_STATE_IN_PROGRESS: The feature status does not fully meet its
        spec at the moment but is trying to meet its spec.
      FEATURE_STATE_READY: The feature status currently meets its spec.
      FEATURE_STATE_INTERNAL_ERROR: The feature status does not fully meet its
        spec at the moment due to an internal error. Contact support.
    """
    FEATURE_STATE_UNSPECIFIED = 0
    FEATURE_STATE_IN_PROGRESS = 1
    FEATURE_STATE_READY = 2
    FEATURE_STATE_INTERNAL_ERROR = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ManagedCaPoolsValue(_messages.Message):
    r"""A map from a region to the status of managed CA pools in that region.

    Messages:
      AdditionalProperty: An additional property for a ManagedCaPoolsValue
        object.

    Fields:
      additionalProperties: Additional properties of type ManagedCaPoolsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ManagedCaPoolsValue object.

      Fields:
        key: Name of the additional property.
        value: A CaPoolsStatus attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('CaPoolsStatus', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  managedCaPools = _messages.MessageField('ManagedCaPoolsValue', 1)
  state = _messages.EnumField('StateValueValuesEnum', 2)


class WorkloadRegistration(_messages.Message):
  r"""Message describing WorkloadRegistration object defines in
  go/workloadcert-registration-api.

  Messages:
    LabelsValue: A LabelsValue object.

  Fields:
    createTime: Output only. [Output only] Create time stamp
    labels: A LabelsValue attribute.
    name: name of resource
    status: Output only. The status of the workloadregistration resource.
    updateTime: Output only. [Output only] Update time stamp
    workloadNamespace: workload_namespace specifies the namespace (which must
      exist in the same project) the workloads are registered to. It's skipped
      for k8s workloads, but required for non-k8s workloads. optional.
    workloadSelector: Criteria used to select the specific group of workloads
      in the registry on which this patch configuration should be applied.
      required.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""A LabelsValue object.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  createTime = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  name = _messages.StringField(3)
  status = _messages.MessageField('RegistrationStatus', 4)
  updateTime = _messages.StringField(5)
  workloadNamespace = _messages.StringField(6)
  workloadSelector = _messages.MessageField('WorkloadSelector', 7)


class WorkloadSelector(_messages.Message):
  r"""WorkloadSelector specifies the criteria used to determine if a rule can
  be applied to a group of workload. The matching criteria is based on
  workload types.

  Fields:
    gcpWorkloadSelector: Selects a group of GCP workloads.
    k8sWorkloadSelector: Selects K8S workloads.
  """

  gcpWorkloadSelector = _messages.MessageField('GCPWorkloadSelector', 1)
  k8sWorkloadSelector = _messages.MessageField('K8SWorkloadSelector', 2)


class WorkloadcertificateProjectsLocationsGetRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class WorkloadcertificateProjectsLocationsGlobalGetWorkloadCertificateFeatureRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsGlobalGetWorkloadCertificateFeatur
  eRequest object.

  Fields:
    name: Required. Name of the `WorkloadCertificateFeature` resource. Format:
      `projects/*/locations/global/workloadCertificateFeature`.
  """

  name = _messages.StringField(1, required=True)


class WorkloadcertificateProjectsLocationsListRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsListRequest object.

  Fields:
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class WorkloadcertificateProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class WorkloadcertificateProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class WorkloadcertificateProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class WorkloadcertificateProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class WorkloadcertificateProjectsLocationsWorkloadRegistrationsCreateRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsWorkloadRegistrationsCreateRequest
  object.

  Fields:
    parent: Required. Value for parent.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and t he request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    workloadRegistration: A WorkloadRegistration resource to be passed as the
      request body.
    workloadRegistrationId: Required. Id of the requesting object If auto-
      generating Id server-side, remove this field and
      workload_registration_id from the method_signature of Create RPC
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  workloadRegistration = _messages.MessageField('WorkloadRegistration', 3)
  workloadRegistrationId = _messages.StringField(4)


class WorkloadcertificateProjectsLocationsWorkloadRegistrationsDeleteRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsWorkloadRegistrationsDeleteRequest
  object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and t he request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class WorkloadcertificateProjectsLocationsWorkloadRegistrationsGetRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsWorkloadRegistrationsGetRequest
  object.

  Fields:
    name: Required. Name of the resource.
  """

  name = _messages.StringField(1, required=True)


class WorkloadcertificateProjectsLocationsWorkloadRegistrationsListRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsWorkloadRegistrationsListRequest
  object.

  Fields:
    filter: Filtering results.
    orderBy: Hint for how to order the results.
    pageSize: Requested page size. Server may return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results the server should return.
    parent: Required. Parent value for ListWorkloadRegistrationsRequest.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class WorkloadcertificateProjectsLocationsWorkloadRegistrationsPatchRequest(_messages.Message):
  r"""A WorkloadcertificateProjectsLocationsWorkloadRegistrationsPatchRequest
  object.

  Fields:
    name: name of resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and t he request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Required. Field mask is used to specify the fields to be
      overwritten in the WorkloadRegistration resource by the update. The
      fields specified in the update_mask are relative to the resource, not
      the full request. A field will be overwritten if it is in the mask. If
      the user does not provide a mask then all fields will be overwritten.
    workloadRegistration: A WorkloadRegistration resource to be passed as the
      request body.
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  updateMask = _messages.StringField(3)
  workloadRegistration = _messages.MessageField('WorkloadRegistration', 4)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a Cloud NetApp Volume Snapshot."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.netapp.volumes.snapshots import client as snapshots_client
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.netapp.volumes.snapshots import flags as snapshots_flags
from googlecloudsdk.command_lib.util.args import labels_util

from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(base.CreateCommand):
  """Creates a Cloud NetApp Volume Snapshot."""

  _RELEASE_TRACK = base.ReleaseTrack.ALPHA

  @staticmethod
  def Args(parser):
    snapshots_flags.AddSnapshotCreateArgs(parser)

  def Run(self, args):
    """Create a Cloud NetApp Volume Snapshot in the current project."""
    snapshot_ref = args.CONCEPTS.snapshot.Parse()

    if args.CONCEPTS.volume.Parse() is None:
      raise exceptions.RequiredArgumentException(
          '--volume', 'Requires a volume to create snapshot of')

    volume_ref = args.CONCEPTS.volume.Parse().RelativeName()
    client = snapshots_client.SnapshotsClient(self._RELEASE_TRACK)
    labels = labels_util.ParseCreateArgs(args,
                                         client.messages.Snapshot.LabelsValue)

    snapshot = client.ParseSnapshotConfig(
        name=snapshot_ref.RelativeName(),
        description=args.description,
        labels=labels)
    result = client.CreateSnapshot(snapshot_ref, volume_ref, args.async_,
                                   snapshot)
    if args.async_:
      command = 'gcloud {} netapp volumes snapshots list'.format(
          self.ReleaseTrack().prefix)
      log.status.Print(
          'Check the status of the new snapshot by listing all snapshots:\n  '
          '$ {} '.format(command))
    return result



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute service-attachment commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags

DEFAULT_LIST_FORMAT = """\
    table(
      name,
      region.basename(),
      targetService.basename(),
      connection_preference
    )"""


class ServiceAttachmentsCompleter(compute_completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(ServiceAttachmentsCompleter, self).__init__(
        collection='compute.serviceAttachments',
        list_command='compute service-attachments list --uri',
        **kwargs)


def AddDescription(parser):
  parser.add_argument(
      '--description',
      help='An optional, textual description for the service attachment.')


def AddConnectionPreference(parser, is_update=False):
  connection_preference_choices = {
      'ACCEPT_AUTOMATIC':
          'Always accept connection requests from consumers automatically.',
      'ACCEPT_MANUAL':
          'Only accept connection requests from consumers with the approval of '
          'the service provider.',
  }

  parser.add_argument(
      '--connection-preference',
      choices=connection_preference_choices,
      type=lambda x: x.replace('-', '_').upper(),
      default=None if is_update else 'ACCEPT_AUTOMATIC',
      help="This defines the service attachment's connection preference.")


def AddEnableProxyProtocolForCreate(parser):
  parser.add_argument(
      '--enable-proxy-protocol',
      action='store_true',
      default=False,
      help="""\
      If True, then enable the proxy protocol which is for supplying client
      TCP/IP address data in TCP connections that traverse proxies on their way
      to destination servers.
      """)


def AddEnableProxyProtocolForUpdate(parser):
  parser.add_argument(
      '--enable-proxy-protocol',
      action=arg_parsers.StoreTrueFalseAction,
      help="""\
      If True, then enable the proxy protocol which is for supplying client
      TCP/IP address data in TCP connections that traverse proxies on their way
      to destination servers.
      """)


def AddDomainNames(parser):
  parser.add_argument(
      '--domain-names',
      type=arg_parsers.ArgList(),
      metavar='DOMAIN_NAMES',
      default=None,
      help="""\
      Specifies a comma separated list of DNS domain names that are used during
      DNS integration on PSC connected endpoints.
      """)


def AddConsumerRejectList(parser):
  parser.add_argument(
      '--consumer-reject-list',
      type=arg_parsers.ArgList(),
      metavar='REJECT_LIST',
      default=None,
      help="""\
      Specifies a comma separated list of projects that are not allowed to
      connect to this service attachment. The project can be specified using its
      id or number.
      """)


def AddConsumerRejectListAlpha(parser):
  parser.add_argument(
      '--consumer-reject-list',
      type=arg_parsers.ArgList(),
      metavar='REJECT_LIST',
      default=None,
      help="""\
      Specifies a comma separated list of projects or networks that are not
      allowed to connect to this service attachment. The project can be
      specified using its project ID or project number and the network can be
      specified using its URL. A given service attachment can manage connections
      at either the project or network level. Therefore, both the reject and
      accept lists for a given service attachment must contain either only
      projects or only networks.""")


def AddConsumerAcceptList(parser):
  parser.add_argument(
      '--consumer-accept-list',
      type=arg_parsers.ArgDict(),
      action='append',
      metavar='PROJECT=LIMIT',
      default=None,
      help="""\
      Adds consumer project(s) with connection limit(s) to the accept list of
      the service attachment.

      For example, `--consumer-accept-list myProjectId1=20` accepts a consumer
      project myProjectId1 with connection limit 20.

      * `PROJECT_ID_OR_NUM` - Consumer project id or number.
      * `CONNECTION_LIMIT` - The max number of allowed connections.
      """)


def AddConsumerAcceptListAlpha(parser):
  parser.add_argument(
      '--consumer-accept-list',
      type=arg_parsers.ArgDict(),
      action='append',
      metavar='PROJECT_OR_NETWORK=LIMIT',
      default=None,
      help="""\
    Specifies which consumer projects or networks are allowed to connect to the
    service attachment. Each project or network has a connection limit. A given
    service attachment can manage connections at either the project or network
    level. Therefore, both the accept and reject lists for a given service
    attachment must contain either only projects or only networks.

    For example, `--consumer-accept-list myProjectId1=20` accepts a consumer
    project myProjectId1 with connection limit 20;
    `--consumer-accept-list projects/myProjectId1/global/networks/myNet1=20`
    accepts a consumer network myNet1 with connection limit 20

    * `PROJECT_OR_NETWORK` - Consumer project ID, project number or network URL.
    * `CONNECTION_LIMIT` - The maximum number of allowed connections.
    """)


def ServiceAttachmentArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='service attachment',
      completer=ServiceAttachmentsCompleter,
      plural=plural,
      required=required,
      regional_collection='compute.serviceAttachments',
      region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Errors for the compute VM instances Ops Agents commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import exceptions

import six


class Error(exceptions.Error):
  """Base exception for Ops Agents exceptions."""


class PolicyError(Error):
  """Base exception for Ops Agents policy exceptions."""


class PolicyMalformedError(PolicyError):
  """Raised when the specified Ops Agents policy is malformed."""

  def __init__(self, policy_id):
    message = (
        'Encountered a malformed policy. The Ops Agents policy [{policy_id}] '
        'may have been modified directly by the OS Config guest policy API / '
        'gcloud commands. If so, please delete and re-create with the Ops '
        'Agents policy gcloud commands. If not, this may be an internal error.'
        .format(policy_id=policy_id))
    super(PolicyMalformedError, self).__init__(message)


class PolicyNotFoundError(PolicyError):
  """Raised when the specified Ops Agents policy is not found."""

  def __init__(self, policy_id):
    message = (
        'Ops Agents policy [{policy_id}] not found.'
        .format(policy_id=policy_id))
    super(PolicyNotFoundError, self).__init__(message)


class PolicyValidationError(PolicyError):
  """Raised when Ops Agents policy validation fails."""


class PolicyValidationMultiError(PolicyValidationError):
  """Raised when multiple Ops Agents policy validations fail."""

  def __init__(self, errors):
    super(PolicyValidationMultiError, self).__init__(
        ' | '.join(six.text_type(error) for error in errors))
    self.errors = errors



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Task for IAM policies on storage resources."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.command_lib.storage import progress_callbacks
from googlecloudsdk.command_lib.storage.tasks import task


class SetIamPolicyTask(task.Task):
  """Sets IAM policy on a storage resource."""

  def __init__(self, url, policy):
    """Initializes task.

    Args:
      url (StorageUrl): Used to identify cloud resource to set policy on.
      policy (object): Provider-specific data type. Currently, only available
        for GCS so Apitools messages.Policy object. If supported for more
        providers in the future, use a generic container.
    """
    super(SetIamPolicyTask, self).__init__()
    self._url = url
    self._policy = policy

  def execute(self, task_status_queue=None):
    """Executes task."""
    client = api_factory.get_api(self._url.scheme)
    if self._url.is_bucket():
      new_policy = client.set_bucket_iam_policy(self._url.bucket_name,
                                                self._policy)
    else:
      new_policy = client.set_object_iam_policy(self._url.bucket_name,
                                                self._url.object_name,
                                                self._policy,
                                                self._url.generation)

    if task_status_queue:
      progress_callbacks.increment_count_callback(task_status_queue)

    return task.Output(
        additional_task_iterators=None,
        messages=[task.Message(task.Topic.SET_IAM_POLICY, payload=new_policy)])

  def __eq__(self, other):
    if not isinstance(other, SetIamPolicyTask):
      return NotImplemented
    return self._url == other._url and self._policy == other._policy
